package com.yunyan.framework.aspectj;

import java.util.ArrayList;
import java.util.List;

import com.yunyan.common.core.domain.BaseEntityVo;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.springframework.stereotype.Component;
import com.yunyan.common.annotation.DataScope;
import com.yunyan.common.core.domain.BaseEntity;
import com.yunyan.common.core.domain.entity.SysRole;
import com.yunyan.common.core.domain.entity.SysUser;
import com.yunyan.common.core.domain.model.LoginUser;
import com.yunyan.common.core.text.Convert;
import com.yunyan.common.utils.SecurityUtils;
import com.yunyan.common.utils.StringUtils;
import com.yunyan.framework.security.context.PermissionContextHolder;

/**
 * 数据过滤处理
 *
 * @author yunyan
 */
@Aspect
@Component
public class DataScopeAspect
{
    /**
     * 全部数据权限
     */
    public static final String DATA_SCOPE_ALL = "1";

    /**
     * 自定数据权限
     */
    public static final String DATA_SCOPE_CUSTOM = "2";

    /**
     * 部门数据权限
     */
    public static final String DATA_SCOPE_DEPT = "3";

    /**
     * 部门及以下数据权限
     */
    public static final String DATA_SCOPE_DEPT_AND_CHILD = "4";

    /**
     * 仅本人数据权限
     */
    public static final String DATA_SCOPE_SELF = "5";

    /**
     * 管理部门查询被管理部门的
     */
    public static final String DATA_SCOPE_DEPT_MANAGE = "8";

    /**
     * 数据权限过滤关键字
     */
    public static final String DATA_SCOPE = "dataScope";

    @Before("@annotation(controllerDataScope)")
    public void doBefore(JoinPoint point, DataScope controllerDataScope) throws Throwable
    {
        clearDataScope(point);
        handleDataScope(point, controllerDataScope);
    }

    protected void handleDataScope(final JoinPoint joinPoint, DataScope controllerDataScope)
    {
        // 获取当前的用户
        LoginUser loginUser = SecurityUtils.getLoginUser();
        //用户的部门, 多部门
        List<Long> deptIdList = loginUser.getDeptIdList();
        if (StringUtils.isNotNull(loginUser))
        {
            SysUser currentUser = loginUser.getUser();
            // 如果是超级管理员，则不过滤数据
            if (StringUtils.isNotNull(currentUser) && !currentUser.isAdmin())
            {
                String permission = StringUtils.defaultIfEmpty(controllerDataScope.permission(), PermissionContextHolder.getContext());
                dataScopeFilter(joinPoint, currentUser, controllerDataScope.deptAlias(),
                        controllerDataScope.userAlias(), permission, deptIdList, controllerDataScope.deptRelationAlias());
            }
        }
    }

    /**
     * 数据范围过滤
     *
     * @param joinPoint 切点
     * @param user 用户
     * @param deptAlias 部门别名
     * @param userAlias 用户别名
     * @param permission 权限字符
     */
    public static void dataScopeFilter(JoinPoint joinPoint, SysUser user, String deptAlias, String userAlias, String permission, List<Long> deptIdList, String deptRelationAlias)
    {
        StringBuilder sqlString = new StringBuilder();
        List<String> conditions = new ArrayList<String>();

        if(deptIdList.isEmpty()) {
            deptIdList.add(-1L);
        }

        for (SysRole role : user.getRoles())
        {
            String dataScope = role.getDataScope();
            if (!DATA_SCOPE_CUSTOM.equals(dataScope) && conditions.contains(dataScope))
            {
                continue;
            }
            if (StringUtils.isNotEmpty(permission) && StringUtils.isNotEmpty(role.getPermissions())
                    && !StringUtils.containsAny(role.getPermissions(), Convert.toStrArray(permission)))
            {
                continue;
            }
            if (DATA_SCOPE_ALL.equals(dataScope))
            {
                sqlString = new StringBuilder();
                break;
            }
            else if (DATA_SCOPE_CUSTOM.equals(dataScope))
            {
                if(StringUtils.isNotBlank(deptRelationAlias)) {
                    sqlString.append(StringUtils.format(
                            " OR EXISTS ( select 1 from sys_user_dept {} where {}.user_id = u.user_id and {}.dept_id IN ( SELECT dept_id FROM sys_role_dept WHERE role_id = {} ) ) ", deptAlias, deptAlias, deptAlias,
                            role.getRoleId()));
                } else {
                    //这个不使用
                    sqlString.append(StringUtils.format(
                            " OR {}.dept_id IN ( SELECT dept_id FROM sys_role_dept WHERE role_id = {} ) ", deptAlias,
                            role.getRoleId()));
                }
            }
            else if (DATA_SCOPE_DEPT.equals(dataScope))
            {
                if(StringUtils.isNotBlank(deptRelationAlias)) {
                    //用户表的多部门
                    for (Long deptId : deptIdList) {
                        sqlString.append(StringUtils.format(" OR EXISTS ( select 1 from sys_user_dept {} where {}.user_id = u.user_id and ({}.dept_id = {}) )", deptAlias, deptAlias, deptAlias, deptId));
                    }
                } else {
                    //用户的多部门
                    for (Long deptId : deptIdList) {
                        sqlString.append(StringUtils.format(" OR {}.dept_id = {} ", deptAlias, deptId));
                    }
                }
            }
            else if (DATA_SCOPE_DEPT_AND_CHILD.equals(dataScope))
            {
                if(StringUtils.isNotBlank(deptRelationAlias)) {
                    //用户表的多部门
                    for (Long deptId : deptIdList) {
                        sqlString.append(StringUtils.format(
                                " OR EXISTS ( select 1 from sys_user_dept {} where {}.user_id = u.user_id and {}.dept_id IN ( SELECT dept_id FROM sys_dept WHERE dept_id = {} or find_in_set( {} , ancestors ) <> 0 ) )",
                                deptAlias, deptAlias, deptAlias, deptId, deptId));
                    }
                } else {
                    //用户的多部门
                    for (Long deptId : deptIdList) {
                        sqlString.append(StringUtils.format(
                                " OR EXISTS ( SELECT dept_id FROM sys_dept WHERE {}.dept_id = dept_id AND (dept_id = {} or find_in_set( {} , ancestors ) <> 0) )",
                                deptAlias, deptId, deptId));
                    }
                }
            }
            else if (DATA_SCOPE_SELF.equals(dataScope))
            {
                if (StringUtils.isNotBlank(userAlias))
                {
                    sqlString.append(StringUtils.format(" OR {}.user_id = {} ", userAlias, user.getUserId()));
                }
                else
                {
                    // 数据权限为仅本人且没有userAlias别名不查询任何数据
                    sqlString.append(StringUtils.format(" OR {}.dept_id = 0 ", deptAlias));
                }
            }
            else if (DATA_SCOPE_DEPT_MANAGE.equals(dataScope))
            {
                //用户的多部门 查询 被管理的部门
                for (Long deptId : deptIdList) {
                    sqlString.append(StringUtils.format(
                            " OR EXISTS ( SELECT dept_id FROM sys_dept_manage WHERE {}.dept_id = dept_id AND (dept_id_manage = {}) )",
                            deptAlias, deptId));
                }
            }
            conditions.add(dataScope);
        }

        if (StringUtils.isNotBlank(sqlString.toString()))
        {
            Object params = joinPoint.getArgs()[0];
            if (StringUtils.isNotNull(params) && params instanceof BaseEntity)
            {
                BaseEntity baseEntity = (BaseEntity) params;
                baseEntity.getParams().put(DATA_SCOPE, " AND (" + sqlString.substring(4) + ")");
            }
            if (StringUtils.isNotNull(params) && params instanceof BaseEntityVo)
            {
                BaseEntityVo baseEntity = (BaseEntityVo) params;
                baseEntity.getParams().put(DATA_SCOPE, " AND (" + sqlString.substring(4) + ")");
            }
        }
    }

    /**
     * 拼接权限sql前先清空params.dataScope参数防止注入
     */
    private void clearDataScope(final JoinPoint joinPoint)
    {
        Object params = joinPoint.getArgs()[0];
        if (StringUtils.isNotNull(params) && params instanceof BaseEntity)
        {
            BaseEntity baseEntity = (BaseEntity) params;
            baseEntity.getParams().put(DATA_SCOPE, "");
        }
        if (StringUtils.isNotNull(params) && params instanceof BaseEntityVo)
        {
            BaseEntityVo baseEntity = (BaseEntityVo) params;
            baseEntity.getParams().put(DATA_SCOPE, "");
        }
    }
}
