package com.yunyan.system.service.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import javax.validation.Validator;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yunyan.common.core.domain.entity.SysDept;
import com.yunyan.common.core.domain.entity.UserDept;
import com.yunyan.system.mapper.*;
import com.yunyan.system.service.IUserDeptService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import com.yunyan.common.annotation.DataScope;
import com.yunyan.common.constant.UserConstants;
import com.yunyan.common.core.domain.entity.SysRole;
import com.yunyan.common.core.domain.entity.SysUser;
import com.yunyan.common.exception.ServiceException;
import com.yunyan.common.utils.SecurityUtils;
import com.yunyan.common.utils.StringUtils;
import com.yunyan.common.utils.bean.BeanValidators;
import com.yunyan.common.utils.spring.SpringUtils;
import com.yunyan.system.domain.SysPost;
import com.yunyan.system.domain.SysUserPost;
import com.yunyan.system.domain.SysUserRole;
import com.yunyan.system.service.ISysConfigService;
import com.yunyan.system.service.ISysUserService;

/**
 * 用户 业务层处理
 *
 * @author yunyan
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements ISysUserService
{
    private static final Logger log = LoggerFactory.getLogger(SysUserServiceImpl.class);

    @Autowired
    private SysUserMapper userMapper;

    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysPostMapper postMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private SysUserPostMapper userPostMapper;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    protected Validator validator;

    @Autowired
    private SysDeptMapper sysDeptMapper;

    @Autowired
    private IUserDeptService userDeptService;

    /**
     * 根据条件分页查询用户列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "up", userAlias = "u", deptRelationAlias = "up")
    public List<SysUser> selectUserList(SysUser user)
    {
        return userMapper.selectUserList(user);
    }

    /**
     * 根据条件分页查询已分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "up")
    public List<SysUser> selectAllocatedList(SysUser user)
    {
        return userMapper.selectAllocatedList(user);
    }

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "up")
    public List<SysUser> selectUnallocatedList(SysUser user)
    {
        return userMapper.selectUnallocatedList(user);
    }

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserByUserName(String userName)
    {
        return userMapper.selectUserByUserName(userName);
    }

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserById(Long userId)
    {
        return userMapper.selectUserById(userId);
    }

    /**
     * 查询用户所属角色组
     *
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserRoleGroup(String userName)
    {
        List<SysRole> list = roleMapper.selectRolesByUserName(userName);
        if (CollectionUtils.isEmpty(list))
        {
            return StringUtils.EMPTY;
        }
        return list.stream().map(SysRole::getRoleName).collect(Collectors.joining(","));
    }

    /**
     * 查询用户所属岗位组
     *
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserPostGroup(String userName)
    {
        List<SysPost> list = postMapper.selectPostsByUserName(userName);
        if (CollectionUtils.isEmpty(list))
        {
            return StringUtils.EMPTY;
        }
        return list.stream().map(SysPost::getPostName).collect(Collectors.joining(","));
    }

    /**
     * 校验用户名称是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public boolean checkUserNameUnique(SysUser user)
    {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        SysUser info = userMapper.checkUserNameUnique(user.getUserName());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验手机号码是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public boolean checkPhoneUnique(SysUser user)
    {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        SysUser info = userMapper.checkPhoneUnique(user.getPhonenumber());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public boolean checkEmailUnique(SysUser user)
    {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        SysUser info = userMapper.checkEmailUnique(user.getEmail());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验用户是否允许操作
     *
     * @param user 用户信息
     */
    @Override
    public void checkUserAllowed(SysUser user)
    {
        if (StringUtils.isNotNull(user.getUserId()) && user.isAdmin())
        {
            throw new ServiceException("不允许操作超级管理员用户");
        }
    }

    /**
     * 校验用户是否有数据权限
     *
     * @param userId 用户id
     */
    @Override
    public void checkUserDataScope(Long userId)
    {
        if (!SysUser.isAdmin(SecurityUtils.getUserId()))
        {
            SysUser user = new SysUser();
            user.setUserId(userId);
            List<SysUser> users = SpringUtils.getAopProxy(this).selectUserList(user);
            if (StringUtils.isEmpty(users))
            {
                throw new ServiceException("没有权限访问用户数据！");
            }
        }
    }

    /**
     * 新增保存用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int insertUser(SysUser user)
    {
        // 新增用户信息
        int rows = userMapper.insertUser(user);
        // 新增用户岗位关联
        insertUserPost(user);
        // 新增用户与角色管理
        insertUserRole(user);
        // 新增用户与部门的关联
        insertUserDept(user);
        return rows;
    }

    /**
     * 注册用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public boolean registerUser(SysUser user)
    {
        return userMapper.insertUser(user) > 0;
    }

    /**
     * 修改保存用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int updateUser(SysUser user)
    {
        Long userId = user.getUserId();
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(userId);
        // 新增用户与角色管理
        insertUserRole(user);
        // 删除用户与岗位关联
        userPostMapper.deleteUserPostByUserId(userId);
        // 新增用户与岗位管理
        insertUserPost(user);
        // 删除用户与部门的关联
        userDeptService.deleteUserDeptByUserId(userId);
        // 新增用户与部门的关联
        insertUserDept(user);

        return userMapper.updateUser(user);
    }

    /**
     * 用户授权角色
     *
     * @param userId 用户ID
     * @param roleIds 角色组
     */
    @Override
    @Transactional
    public void insertUserAuth(Long userId, Long[] roleIds)
    {
        userRoleMapper.deleteUserRoleByUserId(userId);
        insertUserRole(userId, roleIds);
    }

    /**
     * 修改用户状态
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserStatus(SysUser user)
    {
        return userMapper.updateUser(user);
    }

    /**
     * 修改用户基本信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserProfile(SysUser user)
    {
        return userMapper.updateUser(user);
    }

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar 头像地址
     * @return 结果
     */
    @Override
    public boolean updateUserAvatar(String userName, String avatar)
    {
        return userMapper.updateUserAvatar(userName, avatar) > 0;
    }

    /**
     * 重置用户密码
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int resetPwd(SysUser user)
    {
        return userMapper.updateUser(user);
    }

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    @Override
    public int resetUserPwd(String userName, String password)
    {
        return userMapper.resetUserPwd(userName, password);
    }

    /**
     * 新增用户角色信息
     *
     * @param user 用户对象
     */
    public void insertUserRole(SysUser user)
    {
        this.insertUserRole(user.getUserId(), user.getRoleIds());
    }

    /**
     * 新增用户岗位信息
     *
     * @param user 用户对象
     */
    public void insertUserPost(SysUser user)
    {
        Long[] posts = user.getPostIds();
        if (StringUtils.isNotEmpty(posts))
        {
            // 新增用户与岗位管理
            List<SysUserPost> list = new ArrayList<SysUserPost>(posts.length);
            for (Long postId : posts)
            {
                SysUserPost up = new SysUserPost();
                up.setUserId(user.getUserId());
                up.setPostId(postId);
                list.add(up);
            }
            userPostMapper.batchUserPost(list);
        }
    }

    /**
     * 新增用户角色信息
     *
     * @param userId 用户ID
     * @param roleIds 角色组
     */
    public void insertUserRole(Long userId, Long[] roleIds)
    {
        if (StringUtils.isNotEmpty(roleIds))
        {
            // 新增用户与角色管理
            List<SysUserRole> list = new ArrayList<SysUserRole>(roleIds.length);
            for (Long roleId : roleIds)
            {
                SysUserRole ur = new SysUserRole();
                ur.setUserId(userId);
                ur.setRoleId(roleId);
                list.add(ur);
            }
            userRoleMapper.batchUserRole(list);
        }
    }

    /**
     * 保存 用户与部门的关联
     * @param user
     */
    public void insertUserDept(SysUser user)
    {
        if (null != user.getDeptIds() && user.getDeptIds().length > 0)
        {
            // 新增用户与岗位管理
            List<UserDept> list = new ArrayList<>();
            for (Long deptId : user.getDeptIds())
            {
                UserDept up = new UserDept();
                up.setUserId(user.getUserId());
                up.setDeptId(deptId);
                list.add(up);
            }

            userDeptService.saveBatch(list);
        }
    }

    /**
     * 通过用户ID删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserById(Long userId)
    {
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(userId);
        // 删除用户与岗位表
        userPostMapper.deleteUserPostByUserId(userId);
        return userMapper.deleteUserById(userId);
    }

    /**
     * 批量删除用户信息
     *
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserByIds(Long[] userIds)
    {
        for (Long userId : userIds)
        {
            checkUserAllowed(new SysUser(userId));
            checkUserDataScope(userId);
        }
        // 删除用户与角色关联
        userRoleMapper.deleteUserRole(userIds);
        // 删除用户与岗位关联
        userPostMapper.deleteUserPost(userIds);
        return userMapper.deleteUserByIds(userIds);
    }

    /**
     * 导入用户数据
     *
     * @param userList 用户数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName 操作用户
     * @return 结果
     */
    @Override
    public String importUser(List<SysUser> userList, Boolean isUpdateSupport, String operName)
    {
        if (StringUtils.isNull(userList) || userList.size() == 0)
        {
            throw new ServiceException("导入用户数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        String password = configService.selectConfigByKey("sys.user.initPassword");
        for (SysUser user : userList)
        {
            String userName = user.getUserNamePrefix() + user.getUserName();
            user.setUserName(userName);

            try
            {
                //设置部门的导入
                if(StringUtils.isNotBlank(user.getDeptIdsStr())) {
                    String[] deptIdsStr = user.getDeptIdsStr().split(",");
                    Long[] deptIds = StringUtils.stringToLong(deptIdsStr);
                    user.setDeptIds(deptIds);
                }

                //依据 部门区域和编码 确定部门ID
                if(StringUtils.isNotBlank(user.getArea()) && StringUtils.isNotBlank(user.getDeptCode()) ) {
                    LambdaQueryWrapper<SysDept> queryWrapperDept = new LambdaQueryWrapper<>();
                    queryWrapperDept.eq(SysDept::getArea, user.getArea());
                    queryWrapperDept.eq(SysDept::getDeptCode, user.getDeptCode());
                    queryWrapperDept.select(SysDept::getDeptId);
                    List<SysDept> sysDeptList = sysDeptMapper.selectList(queryWrapperDept);
                    if(sysDeptList.size() > 1) {
                        throw new ServiceException("账号导入失败 " + userName +", 依据区域和部门查询到多个部门 [deptCode=" + user.getDeptCode() + ", area=" + user.getArea() + "]");
                    } else if(sysDeptList.size() == 0) {
                        throw new ServiceException("账号导入失败 " + userName +", 依据区域和部门未查询到部门 [deptCode=" + user.getDeptCode() + ", area=" + user.getArea() + "]");
                    }

                    //设置部门的ID
                    user.setDeptIds(new Long[]{sysDeptList.get(0).getDeptId()});
                }

                //设置角色的导入
                if(StringUtils.isNotBlank(user.getRoleIdsStr())) {
                    String[] roleIdsStr = user.getRoleIdsStr().split(",");
                    Long[] roleIds = StringUtils.stringToLong(roleIdsStr);
                    user.setRoleIds(roleIds);
                }


                // 验证是否存在这个用户
                SysUser u = userMapper.selectUserByUserName(userName);
                if (StringUtils.isNull(u))
                {
                    BeanValidators.validateWithException(validator, user);
                    user.setPassword(SecurityUtils.encryptPassword(password));
                    user.setCreateBy(operName);
                    this.insertUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + userName + " 导入成功");
                }
                else if (isUpdateSupport)
                {
                    BeanValidators.validateWithException(validator, user);
                    checkUserAllowed(user);
                    checkUserDataScope(user.getUserId());
                    user.setUpdateBy(operName);
                    this.updateUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + userName + " 更新成功");
                }
                else
                {
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、账号 " + userName + " 已存在");
                }
            }
            catch (Exception e)
            {
                failureNum++;
                String msg = "<br/>" + failureNum + "、账号 " + user.getUserNamePrefix() + user.getUserName() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);
            }
        }
        if (failureNum > 0)
        {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new ServiceException(failureMsg.toString());
        }
        else
        {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }

    /**
     * 通过部门信息, 导入用户数据
     * 不生成的: 部门编码为空的, 部门编码为3位的, 部门单位为主管单位的
     *
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName 操作用户
     * @return 结果
     */
    @Override
    public String importUserByDept(Boolean isUpdateSupport, String operName, String area, String areaPrefix, String notDeptName)
    {
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        String password = configService.selectConfigByKey("sys.user.initPassword");

        //部门区域
        //默认角色 填报员
        Long roleId = 100L;
        /*String area = "田东";
        String areaPrefix = "TD";*/
        LambdaQueryWrapper<SysDept> queryWrapperDept = new LambdaQueryWrapper<>();
        queryWrapperDept.eq(SysDept::getArea, area);
        queryWrapperDept.ne(SysDept::getDeptName, notDeptName);
        queryWrapperDept.select(SysDept::getDeptId, SysDept::getDeptCode, SysDept::getDeptName, SysDept::getArea, SysDept::getDeptLevel);
        queryWrapperDept.last("order by to_number(area_code) ASC, to_number(SUBSTR(dept_code, 1, 3)) ASC, " +
                "to_number(NVL(SUBSTR( dept_code, 4, 3 ), '-1')) ASC, to_number(NVL(SUBSTR(dept_code, 7, 3), '-1')) ASC, " +
                "to_number(NVL(SUBSTR(dept_code, 10, 3), '-1')) ASC, order_num DESC");
        List<SysDept> sysDeptList = sysDeptMapper.selectList(queryWrapperDept);
        //sysDeptList = sysDeptList.subList(0, 2);

        for (SysDept sysDept : sysDeptList) {
            String userName = areaPrefix + sysDept.getDeptCode() + "01";
            String nickName = sysDept.getDeptName();
            Long[] deptIds = new Long[]{sysDept.getDeptId()};
            Long[] roleIds = new Long[]{roleId};

            SysUser user = new SysUser();
            user.setUserName(userName);
            user.setNickName(nickName);
            user.setDeptIds(deptIds);
            user.setRoleIds(roleIds);

            if(StringUtils.isBlank(sysDept.getDeptCode()) || 3 == sysDept.getDeptCode().length() ) {
            // if(StringUtils.isBlank(sysDept.getDeptCode()) || 2 == sysDept.getDeptLevel() ) {
                //failureNum++;
                //failureMsg.append("<br/>" + failureNum + "、账号 " + userName + " 部门编码为: [" + sysDept.getDeptCode() + "]"  + " 部门单位为: [" + sysDept.getDeptLevel() + "]" );
                //continue; //百色市 三位数的也生成
            }

            try
            {
                // 验证是否存在这个用户
                SysUser u = userMapper.selectUserByUserName(userName);
                if (StringUtils.isNull(u))
                {
                    BeanValidators.validateWithException(validator, user);
                    user.setPassword(SecurityUtils.encryptPassword(password));
                    user.setCreateBy(operName);
                    //this.insertUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + userName + " 导入成功");
                }
                else if (isUpdateSupport)
                {
                    BeanValidators.validateWithException(validator, user);
                    checkUserAllowed(user);
                    checkUserDataScope(user.getUserId());
                    user.setUpdateBy(operName);
                    this.updateUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + userName + " 更新成功");
                }
                else
                {
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、账号 " + userName + " 已存在");
                }
            }
            catch (Exception e)
            {
                failureNum++;
                String msg = "<br/>" + failureNum + "、账号 " + user.getUserName() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);
            }
        }


        if (failureNum > 0)
        {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new ServiceException(failureMsg.toString());
        }
        else
        {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }

    /**
     * 通过部门信息, 导入用户数据
     * 不生成的: 部门编码为空的, 部门编码为3位的, 部门单位为主管单位的
     *
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName 操作用户
     * @return 结果
     */
    @Override
    public String importUserByDeptBy3Code(Boolean isUpdateSupport, String operName, String area, String areaPrefix, String notDeptName)
    {
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        String password = configService.selectConfigByKey("sys.user.initPassword");

        //部门区域
        //默认角色 填报员
        //Long roleId = 100L;
        //单位管理员
        Long roleId = 105L;
        /*String area = "田东";
        String areaPrefix = "TD";*/
        LambdaQueryWrapper<SysDept> queryWrapperDept = new LambdaQueryWrapper<>();
        queryWrapperDept.eq(SysDept::getArea, area);
        queryWrapperDept.ne(SysDept::getDeptName, notDeptName);
        queryWrapperDept.select(SysDept::getDeptId, SysDept::getDeptCode, SysDept::getDeptName, SysDept::getArea, SysDept::getDeptLevel);
        queryWrapperDept.last("order by to_number(area_code) ASC, to_number(SUBSTR(dept_code, 1, 3)) ASC, " +
                "to_number(NVL(SUBSTR( dept_code, 4, 3 ), '-1')) ASC, to_number(NVL(SUBSTR(dept_code, 7, 3), '-1')) ASC, " +
                "to_number(NVL(SUBSTR(dept_code, 10, 3), '-1')) ASC, order_num DESC");
        List<SysDept> sysDeptList = sysDeptMapper.selectList(queryWrapperDept);
        //sysDeptList = sysDeptList.subList(0, 2);

        for (SysDept sysDept : sysDeptList) {
            String userName = areaPrefix + sysDept.getDeptCode() + "02";
            String nickName = sysDept.getDeptName() + "审核";
            Long[] deptIds = new Long[]{sysDept.getDeptId()};
            Long[] roleIds = new Long[]{roleId};

            SysUser user = new SysUser();
            user.setUserName(userName);
            user.setNickName(nickName);
            user.setDeptIds(deptIds);
            user.setRoleIds(roleIds);

            if(StringUtils.isBlank(sysDept.getDeptCode()) || 3 != sysDept.getDeptCode().length() ) {
                log.debug("不是3位数, 不保存");
                continue;
            }

            if(StringUtils.isBlank(sysDept.getDeptCode()) || 3 == sysDept.getDeptCode().length() ) {
                // if(StringUtils.isBlank(sysDept.getDeptCode()) || 2 == sysDept.getDeptLevel() ) {
                //failureNum++;
                //failureMsg.append("<br/>" + failureNum + "、账号 " + userName + " 部门编码为: [" + sysDept.getDeptCode() + "]"  + " 部门单位为: [" + sysDept.getDeptLevel() + "]" );
                //continue; //百色市 三位数的也生成
            }

            try
            {
                // 验证是否存在这个用户
                SysUser u = userMapper.selectUserByUserName(userName);
                if (StringUtils.isNull(u))
                {
                    BeanValidators.validateWithException(validator, user);
                    user.setPassword(SecurityUtils.encryptPassword(password));
                    user.setCreateBy(operName);
                    this.insertUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + userName + " 导入成功");
                }
                else if (isUpdateSupport)
                {
                    BeanValidators.validateWithException(validator, user);
                    checkUserAllowed(user);
                    checkUserDataScope(user.getUserId());
                    user.setUpdateBy(operName);
                    this.updateUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + userName + " 更新成功");
                }
                else
                {
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、账号 " + userName + " 已存在");
                }
            }
            catch (Exception e)
            {
                failureNum++;
                String msg = "<br/>" + failureNum + "、账号 " + user.getUserName() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);
            }
        }


        if (failureNum > 0)
        {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new ServiceException(failureMsg.toString());
        }
        else
        {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }
}
